/*

 Nibbles - Copyright (C) 2000 Andrea Fazzi

 This file is part of Nibbles.

 Nibbles is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Nibbles is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Nibbles; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
//---------------------------------------------------------------------------
#include "Vga256.h"

#include <dpmi.h>
#include <string.h>
#include <sys/nearptr.h>
#include <pc.h>
#include <stdio.h>
#include <stdlib.h>

//---------------------------------------------------------------------------
void SetVgaMode(unsigned char mode)
{				// enter selected int 0x10, ah=0 mode

	__dpmi_regs regs;

	memset(&regs, 0, sizeof regs);
	regs.x.ax = mode;
	__dpmi_int(0x10, &regs);

}

//---------------------------------------------------------------------------
unsigned char *GrabScreenPtr()
{				// get pointer to MCGA screen memory

	unsigned char *scrPtr;

	if (__djgpp_nearptr_enable() == 0)
		exit(1);

	scrPtr = (unsigned char *)(0xa0000 + __djgpp_conventional_base);

	return scrPtr;

}

//---------------------------------------------------------------------------
void Release()
{

	__djgpp_nearptr_disable();

}

//---------------------------------------------------------------------------
void PutPixel(unsigned char *ptrBuf, int x, int y, unsigned char color)
{				// write MCGA pixel

	*(ptrBuf + x + (y * 320)) = color;

}

//---------------------------------------------------------------------------
void WaitForRetrace()
{				// wait for NEXT vertical retrace

	char x;			// = inportb(0x3da); // color EGA/VGA input status 1 register

	// for VSYNC paced game: if retrace ongoing, wait for retrace end first
	do {			// while VGA vertical retrace in progress
		x = inportb(0x3da);
	} while (x & 0x8);

	do {			// while VGA vertical retrace not yet in progress
		x = inportb(0x3da);
	} while (!(x & 0x8));

}

//---------------------------------------------------------------------------
void SetPalette(unsigned char *pal)
{				// write palette to VGA hardware

	int i;
	outportb(0x3c8, 0);
	for (i = 0; i < 768; i++) {
		outp(0x3c9, pal[i]);
	}

}

//---------------------------------------------------------------------------
void GetPalette(unsigned char *pal)
{				// read palette from VGA hardware

	int i;
	outportb(0x3c7, 0);
	for (i = 0; i < 768; i++) {
		pal[i] = inportb(0x3c9);
	}

}

//---------------------------------------------------------------------------
void LoadPalette(const char *filename, unsigned char *pal)
{				// read binary palette file

	FILE *palfile;
	memset(&pal[0], 0, 768);

	if (filename != NULL) {
		palfile = fopen(filename, "rb");
		fread(pal, 1, 768, palfile);
		fclose(palfile);
	}

}

//---------------------------------------------------------------------------
void SavePalToTex(const char *filename, unsigned char *pal)
{				// write palette to text file

	FILE *texfile;
	int i;

	texfile = fopen(filename, "wt");
	for (i = 0; i < 255; i += 3) {
		fprintf(texfile, "Color n. %d %d %d %d\n",
			(i / 3), pal[i], pal[i + 1], pal[i + 2]);
	}
	fclose(texfile);

}

//---------------------------------------------------------------------------
bool FadingFX(unsigned char *src_pal, unsigned char *dest_pal, char mode)
{				// update whole palette by 1 step, mode selects style
	// FADING_INTRO: grow values to dest_pal ones
	// FADING_THE_END: decrease values towards zero
	// return true when endpoint reached

	int i;
	int cnt = 0;		// count of palette values which have saturated

	switch (mode) {

	case FADING_INTRO:
		for (i = 0; i < 768; i++) {
			if (src_pal[i] > dest_pal[i]) {
				dest_pal[i]++;
			} else if (src_pal[i] == dest_pal[i]) {
				cnt++;
			}
		}
		break;

	case FADING_THE_END:
		for (i = 0; i < 768; i++) {
			if (dest_pal[i] > 0) {
				dest_pal[i]--;
			} else if (dest_pal[i] == 0) {
				cnt++;
			}
		}
		break;

#if 0
	case FADE_TO_RED:
		for (i = 0; i < 768; i++)
			if ((src_pal[i] < 64) && (i % 3) == 0) {
				dest_pal[i]++;
			} else {
				dest_pal[i]--;
			}
		break;
#endif

	}

	return (cnt >= 768);

}

//---------------------------------------------------------------------------
void ImgColorInfo(const char *filename, unsigned char *img_buf,
		  unsigned int size)
{				// write image histogram to text file

	unsigned int i;
	unsigned int colCnt[256];	// was unsigned char array??
	unsigned char col;

	FILE *texfile;

	for (i = 0; i < 256; i++) {
		colCnt[i] = 0;
	}

	for (i = 0; i < size; i++) {
		col = img_buf[i];
		colCnt[col]++;
	}

	texfile = fopen(filename, "wt");

	for (i = 0; i < 256; i++) {
		fprintf(texfile, "Color n.%d was found %d time(s)\n", i,
			colCnt[i]);
	}

}

//---------------------------------------------------------------------------
