/*

 Nibbles - Copyright (C) 2000 Andrea Fazzi

 This file is part of Nibbles.

 Nibbles is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Nibbles is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Nibbles; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
//---------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Body.h"
#include "DblBuf.h"
#include "AWorm.h"
#include "MathFun.h"

//---------------------------------------------------------------------------
Body::Body()
{
}

//---------------------------------------------------------------------------
Body::~Body()
{
}

//---------------------------------------------------------------------------
void Body::Make(int bodynum, int bodytot)
{

	bodyID = bodynum;
	mass = 8 + (rand() % 42);	// random weight (for death animation)

	direction = DIR_RIGHT;
	memset(&pivotCheck[0], 0, 100);

	if (bodynum > 0) {

		PrevPart = new Body;
		Exists = true;

		if (bodynum == bodytot) {
			type = IS_A_HEAD;
		} else {
			type = IS_A_BODY;
		}

		PrevPart->Make(bodynum - 1, bodytot);

	} else {

		type = IS_A_TAIL;
		Exists = false;

	}

}

//---------------------------------------------------------------------------
void Body::AddPart(Body * prev)
{				// add body segment in current direction

	Exists = true;
	prev->Exists = false;
	prev->bodyID = -1;
	prev->direction = direction;
	prev->mass = 8 + (rand() % 42);	// random weight (for death animation)
	memcpy(&prev->pivotCheck[0], &pivotCheck[0], 100);

	prev->GraphData = new Sprite;
	// prev->GraphData->LoadFrom_SPR_File("art/Body.spr");
	prev->GraphData->W = GraphData->W;
	prev->GraphData->H = GraphData->H;
	prev->GraphData->Size = GraphData->Size;
	prev->GraphData->PtrSprite = (unsigned char *)malloc(GraphData->Size);
	memcpy(prev->GraphData->PtrSprite, GraphData->PtrSprite,
	       GraphData->Size);

	prev->W = prev->GraphData->W;
	prev->H = prev->GraphData->H;

	// update position: no range check needed, all levels have frames
	switch (direction) {

	case DIR_LEFT:
		prev->x = x + 8;
		prev->y = y;
		break;
	case DIR_RIGHT:
		prev->x = x - 8;
		prev->y = y;
		break;
	case DIR_UP:
		prev->x = x;
		prev->y = y + 8;
		break;
	case DIR_DOWN:
		prev->x = x;
		prev->y = y - 8;
		break;

	}

	prev->GraphBuf = GraphBuf;

}

//---------------------------------------------------------------------------
void Body::Grow(int bodynum)
{

	if (bodyID == 0) {

		PrevPart = new Body;
		AddPart(PrevPart);

	}

	bodyID = bodynum;

	if (Exists) {
		PrevPart->Grow(bodynum - 1);
	}

}

//---------------------------------------------------------------------------
void Body::Move(char dir)
{				// update x and y according to direction dir

	switch (dir) {

	case DIR_LEFT:
		x--;
		break;

	case DIR_RIGHT:
		x++;
		break;

	case DIR_DOWN:
		y++;
		break;

	case DIR_UP:
		y--;
		break;

	}

}

//---------------------------------------------------------------------------
bool Delta(int x, int y, int delta)
{				// check if (y - delta) < x < (y + delta)

	return ((x < (y + delta)) && (x > (y - delta)));

}

//---------------------------------------------------------------------------
bool Body::Crawl(pointXY * pivots, unsigned char num, bool StopFlag,
		 bool pivotTest)
{

	unsigned int i;		// was char

	for (i = 0; i < num; i++) {	// loop over pivots

		if ((direction == DIR_LEFT) || (direction == DIR_RIGHT)) {

			if ((x == pivots[i].x) && (Delta(y, pivots[i].y, 3)) &&
			    (pivotCheck[i] == 0)
			    && ((direction + pivots[i].dir) != 0)) {

				direction = pivots[i].dir;
				pivotCheck[i]++;

				pivotTest = (bodyID == 0);

			}
		} else if ((direction == DIR_UP) || (direction == DIR_DOWN)) {

			if ((y == pivots[i].y) && (Delta(x, pivots[i].x, 3)) &&
			    (pivotCheck[i] == 0)
			    && ((direction + pivots[i].dir) != 0)) {

				direction = pivots[i].dir;
				pivotCheck[i]++;

				pivotTest = (bodyID == 0);

			}
		}

	}

	Move(direction);

	if (Exists) {
		PrevPart->Crawl(pivots, num, StopFlag, pivotTest);
	}

	return true;		// return value is never used?
}

//---------------------------------------------------------------------------
void Body::Display()
{

	if (type == IS_A_HEAD) {

		switch (direction) {	// draw worm head in proper orientation

		case DIR_RIGHT:
			GraphData[0].DrawSprite(x, y, GraphBuf->FrameBuf);
			break;
		case DIR_UP:
			GraphData[1].DrawSprite(x - 1, y, GraphBuf->FrameBuf);
			break;
		case DIR_LEFT:
			GraphData[2].DrawSprite(x - 2, y, GraphBuf->FrameBuf);
			break;
		case DIR_DOWN:
			GraphData[3].DrawSprite(x - 1, y, GraphBuf->FrameBuf);
			break;

		}

	} else {		// round body (or tail) segments, no switch/case needed
		GraphData->DrawSprite(x, y, GraphBuf->FrameBuf);
	}

	if (Exists) {
		PrevPart->Display();
	}

}

//---------------------------------------------------------------------------
void Body::Init(LevelHeader * lev, const char *sprname, DblBuf * buf)
{

	int delta;
	int startx, starty, startsize;

	startx = lev->StartX;
	starty = lev->StartY;
	startsize = lev->CurrSize;

	if ((type == IS_A_BODY) || (type == IS_A_TAIL)) {	// body (and tail?) segments are round

		GraphData = new Sprite;
		GraphData->LoadFrom_SPR_File(sprname);
		W = GraphData->W;
		H = GraphData->H;

	} else {		// head segment

		GraphData = new Sprite[4];	// load and rotate sprite to all orientations

		GraphData[0].LoadFrom_SPR_File(sprname);
		W = GraphData[0].W;
		H = GraphData[0].H;

		GraphData[0].Transform(ROTATE_90DEG_LEFT, &GraphData[1]);
		GraphData[0].Transform(ROTATE_180DEG_LEFT, &GraphData[2]);
		GraphData[2].Transform(ROTATE_90DEG_LEFT, &GraphData[3]);

	}

	if (type == IS_A_HEAD) {

		x = startx;
		y = starty;

	} else {

		delta = (startsize - (bodyID)) * 8;

		x = startx + 10 - delta;
		y = starty;

	}

	GraphBuf = buf;

	if (Exists) {

		if (PrevPart->bodyID == 0) {
			PrevPart->Init(lev, "art/Body.spr", buf);
		} else {
			PrevPart->Init(lev, "art/Body.spr", buf);
		}

	}

}

//---------------------------------------------------------------------------
bool Body::ImEatingMySelf(int X, int Y, int w, int h, char dir)
{				// check for self-collision

	int prevX, prevY, prevW, prevH;

	if (!Exists) {
		return false;
	}

	prevX = PrevPart->x;
	prevY = PrevPart->y;
	prevW = PrevPart->W;
	prevH = PrevPart->H;

	if (dir == 0) {

		if ((Range(Y, prevY, (prevY + prevH)) ||
		     Range(Y + h, prevY, (prevY + prevH))) &&
		    Range(X, prevX, (prevX + prevW))) {
			return true;
		}

	} else {		// dir not zero

		if ((Range(X, prevX, (prevX + prevW)) ||
		     Range(X + w, prevX, (prevX + prevW))) &&
		    Range(Y, prevY, (prevY + prevH))) {
			return true;
		}

	}

	return PrevPart->ImEatingMySelf(X, Y, w, h, dir);	// originally no "return", why?

}

//---------------------------------------------------------------------------
void Body::Die(int t)
{

	if (t == 0) {
		y0 = y;
		x0 = x;
	}

	/* death sequence 1, let body parts drop in accelerated movement */

	y = (int)(y0 + 0.4 / mass * (t * t));	// no need to do exact rounding here

	if (Exists) {
		PrevPart->Die(t);
	}

}

//---------------------------------------------------------------------------
bool Body::Eat(int t)
{				// update x and y according to direction

	switch (direction) {

	case DIR_LEFT:
		x--;
		break;
	case DIR_RIGHT:
		x++;
		break;
	case DIR_UP:
		y--;
		break;
	case DIR_DOWN:
		y++;
		break;

	}

	return (t >= 10);

}

//---------------------------------------------------------------------------
