/*

 Nibbles - Copyright (C) 2000 Andrea Fazzi

 This file is part of Nibbles.

 Nibbles is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Nibbles is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Nibbles; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
//---------------------------------------------------------------------------
#include <bios.h>
#include <unistd.h>
#include <string.h>
#include <pc.h>

#include "Vga256.h"
#include "Sprite.h"
#include "Font.h"
#include "Intro.h"

//---------------------------------------------------------------------------
int CntX = 0;
int CntY = 0;
unsigned char pal[768];
unsigned char palFX[768];

Sprite *ptrLogo;
FontBMP *ptrCalisto;

bool flip = true;

const char *pres1 = "Nibbles v0.2 (C) 2000 - 2021 ANDREA FAZZI et al.";
const char *pres2 = "Nibbles comes with ABSOLUTELY NO WARRANTY.";
const char *pres3 = "License: GPL version 2, for details read GPL.txt";
// keyboard instructions added 2021
const char *pres4 = "Steer with cursor keys, leave with ESC key.";

const char *presStrings[4] = { pres1, pres2, pres3, pres4 };

//---------------------------------------------------------------------------
bool PalMorph(int i1, int i2, unsigned char *pal_src, unsigned char *pal_dest)
{
// make pal_dest[i2 + 0..2] approach pal_src[i1 + 0..2] return true on arrival

	int i;
	char cnt = 0;

	for (i = 0; i < 3; i++) {

		if (pal_src[i1 + i] == pal_dest[i2 + i]) {
			cnt++;
		} else {

			if (pal_src[i + i1] > pal_dest[i + i2]) {
				pal_dest[i + i2]++;
			} else {
				pal_dest[i + i2]--;
			}

		}

	}

	return (cnt >= 3);

}

//---------------------------------------------------------------------------
int Logo(DblBuf * DBuffer)
{				// show palette animated title graphics

	bool flip = false;
	bool morph_1 = false;
	bool morph_2 = true;

	int flipCol_1 = 48;
	int flipCol_2 = 48;
	int delta_t = 3 * FADEDELAY;	// will be ignored if flip is true
	int c = 0;

	LoadPalette(NULL, &palFX[0]);
	SetPalette(&palFX[0]);

	// ptrCalisto->GotoXY(100,165);
	// ptrCalisto->Print("press ENTER key...",ptrLogo->PtrSprite);

	DBuffer->ClearBufPat(ptrLogo->PtrSprite);
	DBuffer->Flip();

	while (c != 0x1c0d) {	// 7181 0x1c0d, ENTER to leave intro and start game

		if (kbhit()) {	// if keypress pending, fetch it
			c = bioskey(0);
		}

		if (c == 0x011b) {	// 283 0x011b, ESCAPE
			return EXIT_FROM_LOGO;	// skip intro, then leave game
		}

		if (!flip) {
			flip = FadingFX(&pal[0], &palFX[0], FADING_INTRO);	// fade palette data
		}

		if (flip) {	// animate palette of worm in title screen back and forth

			delta_t = 0;

			if (morph_2) {

				if (PalMorph(84, flipCol_1, &pal[0], &palFX[0])) {
					morph_1 = true;
					morph_2 = false;

					if (flipCol_2 == 84) {
						flipCol_2 = 48;
					} else {
						flipCol_2 = 84;
					}

				}
			}

			if (morph_1) {
				if (PalMorph(48, flipCol_2, &pal[0], &palFX[0])) {
					morph_2 = true;
					morph_1 = false;

					if (flipCol_1 == 48) {
						flipCol_1 = 84;
					} else {
						flipCol_1 = 48;
					}

				}
			}
		}

		SetPalette(&palFX[0]);

		if (delta_t > 0) {
			usleep(delta_t);
		}
		WaitForRetrace();

	}

	return 0;

}

//---------------------------------------------------------------------------
int Intro(DblBuf * DBuffer, unsigned char *scr)
{				// animate title text and logo

	int x, y;

	LoadPalette("art/system.pal", &pal[0]);

	LoadPalette(NULL, &palFX[0]);
	SetPalette(&palFX[0]);

	ptrLogo = new Sprite;
	ptrCalisto = new FontBMP;

	ptrCalisto->Init(256);

	DBuffer->Init(scr);
	DBuffer->ClearBuf(0);

	ptrLogo->LoadFrom_RAW_File("art/logo.raw");
	ptrCalisto->LoadFromFile("font/calisto.fnt");

	// presentation strings, a few center aligned lines of text
	// original locations: centered, Y 85, 100, 115 (3 strings only)

	DBuffer->ClearBuf(0);

	// Palette seems to be RRRG GGBB style
	// draw 3 "worm body" circle patterns
	for (y = 50; y < 150; y++) {
		int yDist = (y - 100) * (y - 100);	// square of vertical distance
		for (x = 0; x < 320; x++) {
			int xDist;
			if (x < 107) {	// left circle
				xDist = x - 53;
			} else if (x > 212) {	// right circle
				xDist = x - 267;
			} else {	// middle circle
				xDist = x - 160;
			}
			int centerDist = (xDist * xDist) + yDist;	// square of dist to nearest center
			unsigned char color = 0;
			if (centerDist < (50 * 50)) {
				color = 4;	// dark green
				if (centerDist < (45 * 45)) {
					color = 8;	// medium green
				}
				PutPixel(DBuffer->FrameBuf, x, y, color);	// buffer, x, y, color
			}	// if more distance: keep background
		}
	}
	// draw "walls" at top and bottom
	for (y = 0; y < 32; y++) {
		for (x = 0; x < 320; x++) {
			unsigned char color = ((x ^ y) << 4) & 0xe0;	// different shades of red
			PutPixel(DBuffer->FrameBuf, x, y, color);	// buffer, x, y, color
			PutPixel(DBuffer->FrameBuf, x, 199 - y, color);	// buffer, x, y, color
		}
	}

	for (y = 0; y < 4; y++) {
		ptrCalisto->GotoXY(160 - ((strlen(presStrings[y]) / 2) * 6),
				   60 + (20 * y));
		ptrCalisto->Print(presStrings[y], DBuffer->FrameBuf);
	}

	DBuffer->Flip();

	// animate palette until user presses a key or animation completed
	while ((!FadingFX(&pal[0], &palFX[0], FADING_INTRO)) && (!kbhit())) {

		WaitForRetrace();
		SetPalette(&palFX[0]);
		usleep(3 * FADEDELAY);

	}

	sleep(2);		// wait 2 seconds

	// animate palette until user presses a key or animation completed
	while ((!FadingFX(&pal[0], &palFX[0], FADING_THE_END)) && (!kbhit())) {

		WaitForRetrace();
		SetPalette(&palFX[0]);
		usleep(3 * FADEDELAY);

	}

	sleep(1);		// wait 1 second

	// nibbles logo
	return (Logo(DBuffer));

}

//---------------------------------------------------------------------------
