/*

 Nibbles - Copyright (C) 2000 Andrea Fazzi

 This file is part of Nibbles.

 Nibbles is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Nibbles is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Nibbles; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
//---------------------------------------------------------------------------
#include <pc.h>
#include <unistd.h>
#include <dos.h>
#include <conio.h>
#include <bios.h>
#include <string.h>
#include <stdlib.h>

#include "Vga256.h"
#include "Sprite.h"
#include "Font.h"
#include "AWorm.h"
#include "Intro.h"

#define NUM_LEVELS 4

//---------------------------------------------------------------------------
int t = 0;			// used for death animation clock in Crawl() calls

unsigned char *scr;
DblBuf *DBuffer;
int currLev = 0;		// current game level

Sprite *LevelsBackGround;
LevelHeader *Levels;

Worm *MyWorm;
FontBMP *ptrFont;

int c = 0;			// most recently seen character from main()

unsigned char pal1[768];
unsigned char pal2[768];
unsigned char palblack[768];

void GeneralInit();
void Free();
void NewLevelInit();
bool NextLevel();

//---------------------------------------------------------------------------
void GeneralInit()
{				// load data files and init graphics

	SetVgaMode(GRAPH256_MODE);
	scr = GrabScreenPtr();
	DBuffer = new DblBuf;

	LoadPalette("art/system.pal", &pal1[0]);
	LoadPalette("art/system.pal", &pal2[0]);
	memset(&palblack[0], 0, 768);

	SetPalette(&pal1[0]);

	ptrFont = new FontBMP;
	ptrFont->Init(255);
	ptrFont->LoadFromFile("font/calisto.fnt");

	LevelsBackGround = new Sprite[NUM_LEVELS];

	LevelsBackGround[0].LoadFrom_RAW_File("levels/Level1.raw");
	LevelsBackGround[1].LoadFrom_RAW_File("levels/Level2.raw");
	LevelsBackGround[2].LoadFrom_RAW_File("levels/Level3.raw");
	LevelsBackGround[3].LoadFrom_RAW_File("levels/Level4.raw");

	Levels = new LevelHeader[NUM_LEVELS];

	Levels[0].LoadFrom_SCP_File("levels/level1.scp");
	Levels[1].LoadFrom_SCP_File("levels/level2.scp");
	Levels[2].LoadFrom_SCP_File("levels/level3.scp");
	Levels[3].LoadFrom_SCP_File("levels/level4.scp");

	DBuffer->Init(scr);

}

//---------------------------------------------------------------------------
void NewLevelInit(int numLev)
{				// start selected level

	int cx;			// X start coordinate for centered level name display
	char *str = Levels[numLev].LevelName;

	cx = 160 - (strlen(str) / 2 * 6);

	MyWorm = new Worm;

	MyWorm->Init(&Levels[numLev], DBuffer);

	DBuffer->ClearBuf(0);
	ptrFont->GotoXY(cx, 100);
	ptrFont->Print(Levels[numLev].LevelName, DBuffer->FrameBuf);
	DBuffer->Flip();

	while (!FadingFX(&pal1[0], &palblack[0], FADING_INTRO)) {
		SetPalette(&palblack[0]);
		usleep(FADEDELAY);
	}

	// FADING_THE_END uses no source palette, so first argument can be NULL
	while (!FadingFX(&palblack[0], &palblack[0], FADING_THE_END)) {
		SetPalette(&palblack[0]);
		usleep(FADEDELAY);
	}

	memset(&palblack[0], 0, 768);

	DBuffer->ClearBuf(0);
	DBuffer->ClearBufPat(LevelsBackGround[currLev].PtrSprite);
	WaitForRetrace();
	DBuffer->Flip();

	while (!FadingFX(&pal1[0], &palblack[0], FADING_INTRO)) {
		SetPalette(&palblack[0]);
		usleep(FADEDELAY);
	}

	memset(&palblack[0], 0, 768);

}

//---------------------------------------------------------------------------
void TryAgain(int numLev)
{				// restart level

	int cx;			// X start coordinate for centered level name display
	char *str = Levels[numLev].LevelName;

	cx = 160 - (strlen(str) / 2 * 6);

	MyWorm = new Worm;

	MyWorm->Init(&Levels[numLev], DBuffer);

	Levels[numLev].WormDeath = false;

	DBuffer->ClearBuf(0);
	ptrFont->GotoXY(cx, 100);
	ptrFont->Print(Levels[numLev].LevelName, DBuffer->FrameBuf);
	DBuffer->Flip();

	while (!FadingFX(&pal1[0], &palblack[0], FADING_INTRO)) {
		SetPalette(&palblack[0]);
		usleep(FADEDELAY);
	}

	// FADING_THE_END uses no source palette, so first argument can be NULL
	while (!FadingFX(&palblack[0], &palblack[0], FADING_THE_END)) {
		SetPalette(&palblack[0]);
		usleep(FADEDELAY);
	}

	memset(&palblack[0], 0, 768);

	if (Levels[numLev].NumLives < 8) {	// draw N worm heads to show available lives
		Levels[numLev].Arts[0].DrawSprite(228 +
						  (11 *
						   (Levels[numLev].NumLives)),
						  189,
						  LevelsBackGround[numLev].
						  PtrSprite);
	}

	DBuffer->ClearBuf(0);
	DBuffer->ClearBufPat(LevelsBackGround[currLev].PtrSprite);

	WaitForRetrace();
	DBuffer->Flip();

	while (!FadingFX(&pal1[0], &palblack[0], FADING_INTRO)) {
		SetPalette(&palblack[0]);
		usleep(FADEDELAY);
	}

	memset(&palblack[0], 0, 768);

}

//---------------------------------------------------------------------------
bool NextLevel()
{				// go to next level (return true if that was possible)

	DBuffer->ClearBuf(0);
	DBuffer->ClearBufPat(LevelsBackGround[currLev].PtrSprite);

	Levels[currLev].Animations(DBuffer);
	DBuffer->Flip();

	// FADING_THE_END does not use a source palette, so that could be NULL here
	while (!FadingFX(&pal1[0], &pal1[0], FADING_THE_END)) {

		SetPalette(&pal1[0]);
		usleep(FADEDELAY);

	}

	memcpy(&pal1[0], &pal2[0], 768);

	if ((currLev + 1) < NUM_LEVELS) {
		currLev++;
		NewLevelInit(currLev);
		return true;
	}

	return false;
}

//---------------------------------------------------------------------------
void Free()
{				// free memory, return to text mode, do hard exit(0) (bad idea)

	delete MyWorm;
	delete DBuffer;
	delete Levels;
	// delete LevelsBackGround;
	delete ptrFont;
	Release();

	SetVgaMode(TEXT_MODE);
	exit(0);

}

//---------------------------------------------------------------------------
void GameOver()
{				// just calls Free()

	Free();

}

//---------------------------------------------------------------------------
int main()
{

	GeneralInit();

	// if ESCAPE pressed before game starts, just skip the game loop now:
	if (Intro(DBuffer, scr) == EXIT_FROM_LOGO) {	// ESCAPE pressed at intro
		c = 283;	// was Free();
	} else {
		NewLevelInit(currLev);
	}

	while (c != 283) {	// 0x11b, ESCAPE

		DBuffer->ClearBuf(0);
		DBuffer->ClearBufPat(LevelsBackGround[currLev].PtrSprite);
		Levels[currLev].Animations(DBuffer);

		MyWorm->Crawl(&Levels[currLev], t);
		MyWorm->Display();

		if (kbhit()) {	// keypress pending

			c = bioskey(0);	// probe for keypress, non-blocking

			switch (c) {

			case 0x4800:	// 18432: // turn north
			case 0x4838:	// with numlock
			case 0x48e0:	// cursor keys
				MyWorm->AddPivot(DIR_UP);
				break;
			case 0x4b00:	// 19200: // turn west
			case 0x4b34:
			case 0x4be0:
				MyWorm->AddPivot(DIR_LEFT);
				break;
			case 0x5000:	// 20480: // turn south
			case 0x5032:
			case 0x50e0:
				MyWorm->AddPivot(DIR_DOWN);
				break;
			case 0x4d00:	// 19712: // turn east
			case 0x4d36:
			case 0x4de0:
				MyWorm->AddPivot(DIR_RIGHT);
				break;

			}
		}

		WaitForRetrace();
		DBuffer->Flip();

		if (Levels[currLev].NumLives < 1) {	// lives exhausted, game over
			// GameOver(); // calls Free() which hard-exits
			break;	// less brutal way to leave the game
		}

		if (Levels[currLev].LevelProgr == Levels[currLev].NumFood) {	// ate all grapes
			if (!NextLevel()) {	// try to go to next level
				break;	// unable to go to next level (last level completed)
			}
		}

		if (Levels[currLev].WormDeath) {	// restart level at death
			TryAgain(currLev);
		}

	}

	Release();
	SetVgaMode(TEXT_MODE);

	return 0;

}

//---------------------------------------------------------------------------
